#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Diagnostics;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;

using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace TestEngineOptimizations
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl _gridGroupingControl1;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItemModifyConfiguration;
		private System.Windows.Forms.MenuItem menuItemSelectConfiguration;
		private System.Windows.Forms.MenuItem menuItemSaveToFile;
		private System.Windows.Forms.MenuItem menuItemOpenFile;
		private System.Windows.Forms.MenuItem menuItemExitApp;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.MenuItem menuItem2;
		public ListboxTextWriter Log;
		private System.Windows.Forms.MenuItem menuItem3;

		ConfigCollection configurations;

		public ConfigCollection Configurations
		{
			get
			{
				return configurations;
			}
			set
			{
				configurations = value;
			}
		}

		internal GridGroupingControl gridGroupingControl1
		{
			get
			{
				return this._gridGroupingControl1;
			}
			set
			{
				if (this._gridGroupingControl1 != value)
				{
					if (this._gridGroupingControl1 != null)
					{
						this._gridGroupingControl1.PropertyChanging -= new DescriptorPropertyChangedEventHandler(_gridGroupingControl1_PropertyChanging);
					}
					this._gridGroupingControl1 = value;
					if (this._gridGroupingControl1 != null)
					{
						this._gridGroupingControl1.PropertyChanging += new DescriptorPropertyChangedEventHandler(_gridGroupingControl1_PropertyChanging);
					}
				}
			}
		}


		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			System.Drawing.Icon ico = new System.Drawing.Icon(Syncfusion.Windows.Forms.WinFormsUtils.FindFile("Icon", "sfgrid.ico"));
			this.Icon = ico ;	

			configurations = new ConfigCollection();
			configurations.Add(new Config("Reset Datasource"));
			configurations.Add(new Config("Flat Virtual List, 30 million records, Virtual Mode"));
			configurations.Add(new Config("Flat Virtual List, VirtualMode"));
			configurations.Add(new Config("Flat Virtual List, No Optimizations"));
			configurations.Add(new Config("Flat Virtual List, Using Custom Engine"));
			configurations.Add(new Config("Flat Virtual List, Sorted"));
			configurations.Add(new Config("Flat Virtual List, Grouped"));
			configurations.Add(new Config("Flat Virtual List, Grouped, VerticalPixelScroll"));
			configurations.Add(new Config("Flat DataView, VirtualMode, Pass-Through sorting enabled."));
			configurations.Add(new Config("Flat DataView, VirtualMode, Pass-Through sorting disabled."));
			configurations.Add(new Config("Flat DataView, Summaries."));
			configurations.Add(new Config("Flat DataView, Grouped."));
////			configurations.Add(new Config());
			

			InitializeSelectMenu();

			gridGroupingControl1 = new GridGroupingControl();
		}

		void InitializeSelectMenu()
		{
			System.EventHandler menuItemSelectConfig_Click = new System.EventHandler(this.menuItemSelectConfig_Click);

			this.menuItemSelectConfiguration.MenuItems.Clear();
			foreach (Config config in configurations)
			{
				MenuItem mi = new MenuItem();
				mi.Text = config.Name;
				mi.Click += menuItemSelectConfig_Click;
				this.menuItemSelectConfiguration.MenuItems.Add(mi);
			}
		}

		private void menuItemSelectConfig_Click(object sender, System.EventArgs e)
		{
			MenuItem mi = (MenuItem) sender;
			Config config = this.configurations.Find(mi.Text);
			Log.WriteLine();
			Log.WriteLine("Select Configuration: " + mi.Text);
			Log.WriteLine(config.Comment);

			int time = Environment.TickCount;

			Cursor.Current = Cursors.WaitCursor;
			config.ApplyTo(this);
			this.Refresh();
			Cursor.Current = Cursors.Arrow;

			Log.WriteLine("Elapsed Time: {0}", Environment.TickCount-time);

		}

		void LogMemoryUsage()
		{
			// Force garbage collection and get used memory size
			GC.Collect();
			System.Threading.Thread.Sleep(10);
			GC.Collect();
			System.Threading.Thread.Sleep(100);
			GC.Collect();
			Log.Write("Optimizations for {0}: ", this.gridGroupingControl1.TableDescriptor.Name);
			Log.Write("VirtualMode: {0}, ", this.gridGroupingControl1.Table.VirtualMode);
			Log.Write("WithoutCounter: {0}, ", this.gridGroupingControl1.Table.WithoutCounter);
			Log.WriteLine("RecordsAsDisplayElements: {0}, ", this.gridGroupingControl1.Table.RecordsAsDisplayElements);
			Process myProcess = Process.GetCurrentProcess();
#if SyncfusionFramework2_0
			double workingSetSizeinKiloBytes = myProcess.WorkingSet64 / 1000;
#else
			double workingSetSizeinKiloBytes = myProcess.WorkingSet / 1000;
#endif
			string s = "Process's physical memory usage: " + workingSetSizeinKiloBytes.ToString() + " kb \r\n";
			Log.WriteLine(s);
		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuItemModifyConfiguration = new System.Windows.Forms.MenuItem();
			this.menuItemSelectConfiguration = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.menuItemSaveToFile = new System.Windows.Forms.MenuItem();
			this.menuItemOpenFile = new System.Windows.Forms.MenuItem();
			this.menuItemExitApp = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem1,
																					  this.menuItem3});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItemModifyConfiguration,
																					  this.menuItemSelectConfiguration,
																					  this.menuItemSaveToFile,
																					  this.menuItemOpenFile,
																					  this.menuItemExitApp});
			this.menuItem1.Text = "Configurations";
			// 
			// menuItemModifyConfiguration
			// 
			this.menuItemModifyConfiguration.Index = 0;
			this.menuItemModifyConfiguration.Text = "&Modify ...";
			this.menuItemModifyConfiguration.Click += new System.EventHandler(this.menuItemModifyConfiguration_Click);
			// 
			// menuItemSelectConfiguration
			// 
			this.menuItemSelectConfiguration.Index = 1;
			this.menuItemSelectConfiguration.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																										this.menuItem2});
			this.menuItemSelectConfiguration.Text = "&Select";
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 0;
			this.menuItem2.Text = "Config";
			// 
			// menuItemSaveToFile
			// 
			this.menuItemSaveToFile.Index = 2;
			this.menuItemSaveToFile.Text = "Save &to ...";
			this.menuItemSaveToFile.Click += new System.EventHandler(this.menuItemSaveToFile_Click);
			// 
			// menuItemOpenFile
			// 
			this.menuItemOpenFile.Index = 3;
			this.menuItemOpenFile.Text = "&Open ...";
			this.menuItemOpenFile.Click += new System.EventHandler(this.menuItemOpenFile_Click);
			// 
			// menuItemExitApp
			// 
			this.menuItemExitApp.Index = 4;
			this.menuItemExitApp.Text = "&Exit";
			this.menuItemExitApp.Click += new System.EventHandler(this.menuItemExitApp_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 1;
			this.menuItem3.Text = "About";
			this.menuItem3.Click += new System.EventHandler(this.menuItem3_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(664, 366);
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.Text = "EngineOptimizations...";

		}
		#endregion

		Timer t = null;

		private void _gridGroupingControl1_PropertyChanging(object sender, DescriptorPropertyChangedEventArgs e)
		{
			if (Log != null)
			{
				Log.WriteLine(e.ToString());
				if (t != null)
				{
					t.Tick -= new EventHandler(t_Tick);
					t.Dispose();
				}
				t = new Timer();
				t.Interval = 100;
				t.Tick += new EventHandler(t_Tick);
				t.Start();
			}
		}

		private void t_Tick(object sender, EventArgs e)
		{
			Timer t = (Timer) sender;
			t.Tick -= new EventHandler(t_Tick);
			t.Dispose();
			this.LogMemoryUsage();
		}

		private void menuItemModifyConfiguration_Click(object sender, System.EventArgs e)
		{
			System.ComponentModel.Design.CollectionEditor ce = new System.ComponentModel.Design.CollectionEditor(this.configurations.GetType());
			Syncfusion.ComponentModel.WindowsFormsEditorServiceContainer esc = new Syncfusion.ComponentModel.WindowsFormsEditorServiceContainer(null);
			PropertyDescriptor pd = TypeDescriptor.GetProperties(this)["Configurations"];
			Syncfusion.ComponentModel.TypeDescriptorContext tdc = new Syncfusion.ComponentModel.TypeDescriptorContext(this, pd);
			tdc.ServiceProvider = esc;

			ce.EditValue(tdc, esc, Configurations);
		}

		private void menuItemExitApp_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		static XmlSerializer configCollectionSerializer;

		static XmlSerializer ConfigCollectionSerializer
		{
			get
			{
				if (configCollectionSerializer == null)
					configCollectionSerializer = new XmlSerializer(typeof(ConfigCollection));
				return configCollectionSerializer;
			}
		}

		private void menuItemOpenFile_Click(object sender, System.EventArgs e)
		{
			try
			{
				FileDialog dlg = new OpenFileDialog();
				dlg.DefaultExt = "xml";
				if (dlg.ShowDialog() == DialogResult.OK)
				{
					XmlTextReader xr = new XmlTextReader(dlg.FileName);
					this.configurations = (ConfigCollection) ConfigCollectionSerializer.Deserialize(xr);
					xr.Close();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
		}

		private void menuItemSaveToFile_Click(object sender, System.EventArgs e)
		{
			try
			{
				FileDialog dlg = new SaveFileDialog();
				dlg.DefaultExt = "xml";
				if (dlg.ShowDialog() == DialogResult.OK)
				{
					XmlTextWriter xw = new XmlTextWriter(dlg.FileName, System.Text.Encoding.UTF8);
					xw.Formatting = Formatting.Indented;
					ConfigCollectionSerializer.Serialize(xw, configurations);
					xw.Close();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}

		}

		private void menuItem3_Click(object sender, System.EventArgs e)
		{
//			DemoCommon.AboutForm abtForm = new DemoCommon.AboutForm(AppDomain.CurrentDomain.GetAssemblies());
//			abtForm.ShowDialog();
		}
	}

}
