#region Copyright Syncfusion Inc. 2001 - 2006
//
//  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

namespace TraderGridTest
{

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		internal Timer timer;
		private DataTable table;
		private GridGroupingControl gridGroupingControl1;
		private System.Random rand;
		private int m_numUpdatesPerTick = 10;
		DataSet ds;

		private int toggleInsertRemove = 10;// toggle between inserting and removing after n inserts/n removals.
		private int insertRemoveModulus = 5; // frequency of insert / remove : every n timer ticks
		int initialRowCount = 100;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.CheckBox checkBoxFilter;
		private System.Windows.Forms.TextBox textBoxFilter;
		private System.Windows.Forms.CheckBox checkBoxGrouping;
		private System.Windows.Forms.Button buttonApplyFilter;
		private System.Windows.Forms.Label labelTimerInterval;
		private System.Windows.Forms.TrackBar trackBarTimer;
		private System.Windows.Forms.CheckBox checkBoxSorting;
		private System.Windows.Forms.CheckBox checkBoxColor;
		private System.Windows.Forms.Panel panel2;
		private System.Windows.Forms.Button button2;
		private System.Windows.Forms.CheckBox checkBox1;
		private System.Windows.Forms.TextBox textBox1;
		private System.Windows.Forms.CheckBox checkBox2;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TrackBar trackBar1;
		private System.Windows.Forms.CheckBox checkBox3;
		private System.Windows.Forms.CheckBox checkBox4;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.TrackBar trackBarBlinkFrequency;
		private System.Windows.Forms.Label labelBlinkTime;

		// Alter this integer to change the type of test
		int insertRemoveCount = 2;                      // use 1 if you want to check out inserting and removing rows

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Form settings
			System.Drawing.Icon ico = new System.Drawing.Icon(Syncfusion.Windows.Forms.WinFormsUtils.FindFile("Icon", "sfgrid.ico"));
			this.Icon = ico;
			this.Size = new Size(800, 600);

			// Setup the grid
			gridGroupingControl1 = new GridGroupingControl();
			gridGroupingControl1.VerticalThumbTrack = true;
			gridGroupingControl1.HorizontalThumbTrack = true;
			gridGroupingControl1.TableOptions.VerticalPixelScroll = true;
			gridGroupingControl1.Location = new Point(0,0);
			gridGroupingControl1.Size = new Size(this.ClientSize.Width, this.panel1.Top);
			gridGroupingControl1.Anchor = AnchorStyles.Left|AnchorStyles.Top|AnchorStyles.Right|AnchorStyles.Bottom;
			this.Controls.Add(gridGroupingControl1);

            gridGroupingControl1.TableOptions.ListBoxSelectionMode = SelectionMode.MultiExtended;

			OptimizeGrid();
            
			gridGroupingControl1.AllowSwapDataViewWithDataTableList = false; // we want to use "InsertAt" logice with .NET 2.0 ...

			// Setup datatable with random data
			InitializeDataTable();
			gridGroupingControl1.DataSource = table;

			// Highlight up and downticks
			gridGroupingControl1.BlinkTime = 700;  // in ms
			for (int i = 1; i <= 20; i++)
				gridGroupingControl1.TableDescriptor.Columns[i.ToString()].AllowBlink = true;

			gridGroupingControl1.Engine.AddBaseStylesForBlinking();

			//gridGroupingControl1.BaseStyles[GridEngine.BlinkIncreased].StyleInfo.Font.Bold = true;
			gridGroupingControl1.BaseStyles[GridEngine.BlinkIncreased].StyleInfo.TextColor = Color.White;

			//gridGroupingControl1.BaseStyles[GridEngine.BlinkIncreased].StyleInfo.Font.Bold = true;
			gridGroupingControl1.BaseStyles[GridEngine.BlinkReduced].StyleInfo.TextColor = Color.White;

			gridGroupingControl1.Engine.BaseStyles.Add("CustomStyle");
			//gridGroupingControl1.BaseStyles["CustomStyle"].StyleInfo.Font.Bold = true;
			gridGroupingControl1.BaseStyles["CustomStyle"].StyleInfo.TextColor = Color.Black;
			gridGroupingControl1.BaseStyles["CustomStyle"].StyleInfo.BackColor = Color.White;

			this.gridGroupingControl1.TableControlPrepareViewStyleInfo += new GridTableControlPrepareViewStyleInfoEventHandler(gridGroupingControl1_TableControlPrepareViewStyleInfo);

			gridGroupingControl1.TableDescriptor.Columns[0].MaxLength = 8;
			gridGroupingControl1.TableDescriptor.Columns[5].MaxLength = 6;
			gridGroupingControl1.TableDescriptor.Columns[9].MaxLength = 5;
			gridGroupingControl1.TableDescriptor.Columns[14].MaxLength = 6;
			gridGroupingControl1.TableDescriptor.Columns["1"].MaxLength = 8;

			// Sort by column "1"
			gridGroupingControl1.TableDescriptor.SortedColumns.Add("1");

			//Optional: Filter records
			//gridGroupingControl1.TableDescriptor.RecordFilters.Add("[2] > 88");

			Console.WriteLine(gridGroupingControl1.TableDescriptor.Columns["1"].MaxLength);

			this.textBoxFilter.Text = "[2] > 88";
			this.gridGroupingControl1.PropertyChanged += new DescriptorPropertyChangedEventHandler(gridGroupingControl1_PropertyChanged);
			this.gridGroupingControl1.ShowGroupDropArea = true;

          // this.gridGroupingControl1.TableControl.EnableDoubleBufferSurface();

			// Timer
			SetupTimer();
		}

		void gridGroupingControl1_TableControlPrepareViewStyleInfo(object sender, GridTableControlPrepareViewStyleInfoEventArgs e)
		{
			GridTableCellStyleInfo style = (GridTableCellStyleInfo)e.Inner.Style;
			BlinkState bs = gridGroupingControl1.GetBlinkState(style.TableCellIdentity);

			if (bs != BlinkState.None)
			{
				if (bs == BlinkState.NewRecord)
				{
					e.Inner.Style.BaseStyle = "CustomStyle";
					//e.Inner.Style.BackColor = Color.White;
					//e.Inner.Style.TextColor = Color.Black;
				}
			}
		}

		private void OptimizeGrid()
		{
			// Optimize memory footprint, drawing and ListChanged performance

			// Use less memory for internal binary tree structures
			gridGroupingControl1.CounterLogic = EngineCounters.YAmount;
			gridGroupingControl1.AllowedOptimizations = EngineOptimizations.DisableCounters | EngineOptimizations.RecordsAsDisplayElements;

			// Affected settings by UseDefaultsForFasterDrawing are: InvalidateAllWhenListChanged, 
			// TableOptions.ColumnsMaxLengthStrategy, TableOptions.GridLineBorder, 
			// TableOptions.DrawTextWithGdiInterop, TableOptions.VerticalPixelScroll,
			// Appearance.AnyRecordFieldCell.WrapText and Appearance.AnyRecordFieldCell.Trimming.
			gridGroupingControl1.UseDefaultsForFasterDrawing = true;

			// Instruct grouping control not to access datasource through
			// CurrencyManager API and instead access list diretly. When set to
			// false you wil however loose support for built-in CurrencyManager 
			// synchronization, so only do this if you don't need CurrencyManager synchronization.
			gridGroupingControl1.BindToCurrencyManager = false;

			// Update display with changes, optionally use ScrollWindow for insert and removing rows.
			//gridGroupingControl1.UpdateDisplayFrequency = 0;  // only manual updates (when you call grid.Update)
			gridGroupingControl1.UpdateDisplayFrequency = 1;  // immediate update after each ListChanged.
			//gridGroupingControl1.UpdateDisplayFrequency = 100;  // update every 100 ms.
			gridGroupingControl1.InsertRemoveBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate;  // Scrollwindow will cause immediate update.
			gridGroupingControl1.SortPositionChangedBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate;  // Scrollwindow will cause immediate update.

			// Don't forget this property. If it is true then InsertRemoveBehavior and SortPositionChangedBehavior
			// properties would have no effect at all. Its default value is false if UseDefaultsForFasterDrawing = true
			// has been set, but setting it explictly makes sure this is false.
			gridGroupingControl1.InvalidateAllWhenListChanged = false;
		}

		private void SetupTimer()
		{
			timer = new Timer();
			timer.Tick += new EventHandler(timer_Tick);
			timer.Interval = 100;
			timer.Start();

			this.UpdateDialog();

			this.gridGroupingControl1.Focus();
		}

		void InitializeDataTable()
		{
            ds = new DataSet();
			table = new DataTable("RandomData");
			table.Columns.Add("Product", typeof(string));
			table.Columns.Add("1", typeof(System.Double));
			table.Columns.Add("2", typeof(System.Double));
			table.Columns.Add("3", typeof(System.Double));
			table.Columns.Add("4", typeof(System.Double));
			table.Columns.Add("5", typeof(System.Double));
			table.Columns.Add("6", typeof(System.Double));
			table.Columns.Add("7", typeof(System.Double));
			table.Columns.Add("8", typeof(System.Double));
			table.Columns.Add("9", typeof(System.Double));
			table.Columns.Add("10", typeof(System.Double));
			table.Columns.Add("11", typeof(System.Double));
			table.Columns.Add("12", typeof(System.Double));
			table.Columns.Add("13", typeof(System.Double));
			table.Columns.Add("14", typeof(System.Double));
			table.Columns.Add("15", typeof(System.Double));
			table.Columns.Add("16", typeof(System.Double));
			table.Columns.Add("17", typeof(System.Double));
			table.Columns.Add("18", typeof(System.Double));
			table.Columns.Add("19", typeof(System.Double));
			table.Columns.Add("20", typeof(System.Double));
            ds.Tables.Add(table);

			rand = new Random(0);

			for (int i = 0; i < initialRowCount; i++)
			{
				int next = rand.Next(100);
				table.Rows.Add(new object[]{"P" + i.ToString("00000"), next+1,next+2,
											   next+3,next+4,next+5,next+6,
											   next+7,next+8,next+9,next+10,
											   next+11,next+12,next+13,next+14,
											   next+15,next+16,next+17,
											   next+18,next+19,next+20});
			}

		}

		void gridGroupingControl1_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
		{
			GridTableCellStyleInfo style = (GridTableCellStyleInfo)e.Style;
			if (e.TableCellIdentity.TableCellType == GridTableCellType.RecordFieldCell
				|| e.TableCellIdentity.TableCellType == GridTableCellType.AlternateRecordFieldCell)
			{
				//if (this.gridGroupingControl1.GetBlinkState(e.TableCellIdentity) == BlinkState.NewRecord)
				//{
				//    e.Style.BaseStyle = "CustomStyle";
				//    e.Style.TextColor = Color.Yellow;
				//}

				if (e.TableCellIdentity.Column.FieldDescriptor.FieldPropertyType == typeof(string))
					return;

				//if (this.gridGroupingControl1.GetBlinkState(e.TableCellIdentity) == BlinkState.None)
			{
				// Get the value from column "1" and color all cells in record based
				// on this value.
				Record r = e.Style.TableCellIdentity.DisplayElement.GetRecord();
				object value = r.GetValue("1");
				int v = Convert.ToInt32(value) % colors.Length;
				e.Style.BackColor = colors[v];
			}

			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.panel1 = new System.Windows.Forms.Panel();
			this.buttonApplyFilter = new System.Windows.Forms.Button();
			this.checkBoxGrouping = new System.Windows.Forms.CheckBox();
			this.textBoxFilter = new System.Windows.Forms.TextBox();
			this.checkBoxFilter = new System.Windows.Forms.CheckBox();
			this.labelTimerInterval = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.trackBarTimer = new System.Windows.Forms.TrackBar();
			this.checkBoxSorting = new System.Windows.Forms.CheckBox();
			this.checkBoxColor = new System.Windows.Forms.CheckBox();
			this.label4 = new System.Windows.Forms.Label();
			this.trackBarBlinkFrequency = new System.Windows.Forms.TrackBar();
			this.labelBlinkTime = new System.Windows.Forms.Label();
			this.panel2 = new System.Windows.Forms.Panel();
			this.button2 = new System.Windows.Forms.Button();
			this.checkBox1 = new System.Windows.Forms.CheckBox();
			this.textBox1 = new System.Windows.Forms.TextBox();
			this.checkBox2 = new System.Windows.Forms.CheckBox();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.trackBar1 = new System.Windows.Forms.TrackBar();
			this.checkBox3 = new System.Windows.Forms.CheckBox();
			this.checkBox4 = new System.Windows.Forms.CheckBox();
			this.panel1.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.trackBarTimer)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.trackBarBlinkFrequency)).BeginInit();
			this.panel2.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.trackBar1)).BeginInit();
			this.SuspendLayout();
			// 
			// panel1
			// 
			this.panel1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panel1.Controls.Add(this.buttonApplyFilter);
			this.panel1.Controls.Add(this.checkBoxGrouping);
			this.panel1.Controls.Add(this.textBoxFilter);
			this.panel1.Controls.Add(this.checkBoxFilter);
			this.panel1.Controls.Add(this.labelTimerInterval);
			this.panel1.Controls.Add(this.label1);
			this.panel1.Controls.Add(this.trackBarTimer);
			this.panel1.Controls.Add(this.checkBoxSorting);
			this.panel1.Controls.Add(this.checkBoxColor);
			this.panel1.Controls.Add(this.label4);
			this.panel1.Controls.Add(this.trackBarBlinkFrequency);
			this.panel1.Controls.Add(this.labelBlinkTime);
			this.panel1.Location = new System.Drawing.Point(-8, 368);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(704, 184);
			this.panel1.TabIndex = 0;
			// 
			// buttonApplyFilter
			// 
			this.buttonApplyFilter.Location = new System.Drawing.Point(400, 56);
			this.buttonApplyFilter.Name = "buttonApplyFilter";
			this.buttonApplyFilter.Size = new System.Drawing.Size(80, 24);
			this.buttonApplyFilter.TabIndex = 8;
			this.buttonApplyFilter.Text = "Apply";
			this.buttonApplyFilter.Click += new System.EventHandler(this.buttonApplyFilter_Click);
			// 
			// checkBoxGrouping
			// 
			this.checkBoxGrouping.Location = new System.Drawing.Point(32, 96);
			this.checkBoxGrouping.Name = "checkBoxGrouping";
			this.checkBoxGrouping.Size = new System.Drawing.Size(112, 24);
			this.checkBoxGrouping.TabIndex = 7;
			this.checkBoxGrouping.Text = "Enable Grouping";
			this.checkBoxGrouping.CheckedChanged += new System.EventHandler(this.checkBoxGrouping_CheckedChanged);
			// 
			// textBoxFilter
			// 
			this.textBoxFilter.Location = new System.Drawing.Point(160, 56);
			this.textBoxFilter.Name = "textBoxFilter";
			this.textBoxFilter.Size = new System.Drawing.Size(216, 20);
			this.textBoxFilter.TabIndex = 6;
			this.textBoxFilter.Text = "textBox1";
			// 
			// checkBoxFilter
			// 
			this.checkBoxFilter.Location = new System.Drawing.Point(32, 56);
			this.checkBoxFilter.Name = "checkBoxFilter";
			this.checkBoxFilter.Size = new System.Drawing.Size(96, 24);
			this.checkBoxFilter.TabIndex = 5;
			this.checkBoxFilter.Text = "Enable Filter";
			this.checkBoxFilter.CheckedChanged += new System.EventHandler(this.checkBoxFilter_CheckedChanged);
			// 
			// labelTimerInterval
			// 
			this.labelTimerInterval.Location = new System.Drawing.Point(368, 8);
			this.labelTimerInterval.Name = "labelTimerInterval";
			this.labelTimerInterval.Size = new System.Drawing.Size(112, 40);
			this.labelTimerInterval.TabIndex = 4;
			this.labelTimerInterval.Text = "label2";
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(152, 48);
			this.label1.TabIndex = 3;
			this.label1.Text = "Timer Frequency";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// trackBarTimer
			// 
			this.trackBarTimer.Location = new System.Drawing.Point(160, 8);
			this.trackBarTimer.Name = "trackBarTimer";
			this.trackBarTimer.Size = new System.Drawing.Size(184, 50);
			this.trackBarTimer.TabIndex = 2;
			this.trackBarTimer.Scroll += new System.EventHandler(this.trackBarTimer_Scroll);
			// 
			// checkBoxSorting
			// 
			this.checkBoxSorting.Location = new System.Drawing.Point(160, 96);
			this.checkBoxSorting.Name = "checkBoxSorting";
			this.checkBoxSorting.Size = new System.Drawing.Size(112, 24);
			this.checkBoxSorting.TabIndex = 7;
			this.checkBoxSorting.Text = "Enable Sorting";
			this.checkBoxSorting.CheckedChanged += new System.EventHandler(this.checkBoxSorting_CheckedChanged);
			// 
			// checkBoxColor
			// 
			this.checkBoxColor.Location = new System.Drawing.Point(296, 96);
			this.checkBoxColor.Name = "checkBoxColor";
			this.checkBoxColor.Size = new System.Drawing.Size(112, 24);
			this.checkBoxColor.TabIndex = 7;
			this.checkBoxColor.Text = "Enable Coloring";
			this.checkBoxColor.CheckedChanged += new System.EventHandler(this.checkBoxColor_CheckedChanged);
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label4.Location = new System.Drawing.Point(8, 136);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(152, 48);
			this.label4.TabIndex = 3;
			this.label4.Text = "Blink Frequency";
			this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// trackBarBlinkFrequency
			// 
			this.trackBarBlinkFrequency.Location = new System.Drawing.Point(160, 136);
			this.trackBarBlinkFrequency.Maximum = 20;
			this.trackBarBlinkFrequency.Name = "trackBarBlinkFrequency";
			this.trackBarBlinkFrequency.Size = new System.Drawing.Size(184, 50);
			this.trackBarBlinkFrequency.SmallChange = 2;
			this.trackBarBlinkFrequency.TabIndex = 2;
			this.trackBarBlinkFrequency.TickFrequency = 2;
			this.trackBarBlinkFrequency.Scroll += new System.EventHandler(this.trackBarBlinkFrequency_Scroll);
			// 
			// labelBlinkTime
			// 
			this.labelBlinkTime.Location = new System.Drawing.Point(368, 136);
			this.labelBlinkTime.Name = "labelBlinkTime";
			this.labelBlinkTime.Size = new System.Drawing.Size(112, 40);
			this.labelBlinkTime.TabIndex = 4;
			this.labelBlinkTime.Text = "Disabled.";
			// 
			// panel2
			// 
			this.panel2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panel2.Controls.Add(this.button2);
			this.panel2.Controls.Add(this.checkBox1);
			this.panel2.Controls.Add(this.textBox1);
			this.panel2.Controls.Add(this.checkBox2);
			this.panel2.Controls.Add(this.label2);
			this.panel2.Controls.Add(this.label3);
			this.panel2.Controls.Add(this.trackBar1);
			this.panel2.Controls.Add(this.checkBox3);
			this.panel2.Controls.Add(this.checkBox4);
			this.panel2.Location = new System.Drawing.Point(-8, 416);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(704, 216);
			this.panel2.TabIndex = 0;
			// 
			// button2
			// 
			this.button2.Location = new System.Drawing.Point(504, 56);
			this.button2.Name = "button2";
			this.button2.Size = new System.Drawing.Size(80, 24);
			this.button2.TabIndex = 8;
			this.button2.Text = "Apply";
			// 
			// checkBox1
			// 
			this.checkBox1.Location = new System.Drawing.Point(32, 88);
			this.checkBox1.Name = "checkBox1";
			this.checkBox1.Size = new System.Drawing.Size(112, 24);
			this.checkBox1.TabIndex = 7;
			this.checkBox1.Text = "Enable Grouping";
			// 
			// textBox1
			// 
			this.textBox1.Location = new System.Drawing.Point(160, 56);
			this.textBox1.Name = "textBox1";
			this.textBox1.Size = new System.Drawing.Size(320, 20);
			this.textBox1.TabIndex = 6;
			this.textBox1.Text = "textBox1";
			// 
			// checkBox2
			// 
			this.checkBox2.Location = new System.Drawing.Point(32, 56);
			this.checkBox2.Name = "checkBox2";
			this.checkBox2.Size = new System.Drawing.Size(96, 24);
			this.checkBox2.TabIndex = 5;
			this.checkBox2.Text = "Enable Filter";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(368, 8);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(112, 40);
			this.label2.TabIndex = 4;
			this.label2.Text = "label2";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label3.Location = new System.Drawing.Point(8, 8);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(152, 48);
			this.label3.TabIndex = 3;
			this.label3.Text = "Timer Frequency";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// trackBar1
			// 
			this.trackBar1.Location = new System.Drawing.Point(160, 8);
			this.trackBar1.Name = "trackBar1";
			this.trackBar1.Size = new System.Drawing.Size(184, 50);
			this.trackBar1.TabIndex = 2;
			// 
			// checkBox3
			// 
			this.checkBox3.Location = new System.Drawing.Point(160, 88);
			this.checkBox3.Name = "checkBox3";
			this.checkBox3.Size = new System.Drawing.Size(112, 24);
			this.checkBox3.TabIndex = 7;
			this.checkBox3.Text = "Enable Sorting";
			// 
			// checkBox4
			// 
			this.checkBox4.Location = new System.Drawing.Point(296, 88);
			this.checkBox4.Name = "checkBox4";
			this.checkBox4.Size = new System.Drawing.Size(112, 24);
			this.checkBox4.TabIndex = 7;
			this.checkBox4.Text = "Enable Coloring";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(696, 546);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.panel2);
			this.Name = "Form1";
			this.Text = "TraderGridTest...";
			this.panel1.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.trackBarTimer)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.trackBarBlinkFrequency)).EndInit();
			this.panel2.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.trackBar1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.Run(new Form1());
		}

		int icount = 0;
		int timerCount = 0;
		int ti = 0;
		bool shouldInsert = false;
        bool waitKey = false;

        void WaitForKey()
        {
            skipTimer = true;
            timer.Tick -= new EventHandler(timer_Tick);
            timer.Enabled = false;
            Application.DoEvents();
            if (Control.ModifierKeys == Keys.Shift)
                waitKey = true;

            if (waitKey)
            {
                while (Control.ModifierKeys == Keys.None)
                {
                    System.Threading.Thread.Sleep(5);
                    Application.DoEvents();
                }
            }
            if (!Visible)
                Application.Exit();
            timer.Tick += new EventHandler(timer_Tick);
            timer.Enabled = true;
            skipTimer = false;
        }

        bool skipTimer = false;

		private void timer_Tick(object sender, EventArgs e)
		{
            if (skipTimer)
                return;

			timerCount++;

			try
			{
				for (int i = 0; i < m_numUpdatesPerTick; i++)
				{
                  //  Application.DoEvents();

					int recNum = rand.Next(table.Rows.Count - 1);
					int rowNum = recNum + 1;
					int col = rand.Next(16) + 1;
					int colNum = col + 1;
					DataRow drow = table.Rows[recNum];
					if (!(drow[col] is DBNull))
						drow[col] = (int) (Convert.ToDouble(drow[col]) * (rand.Next(50) / 100.0f + 0.8));// rand.Next(100);
					//drow[col] = (int) rand.Next(16) + 18; //(Convert.ToDouble(drow[col]) * (rand.Next(50) / 100.0f + 0.8));// rand.Next(100);
					//					            if (timerCount % 2 == 1)
					//									drow[col] = 24;//(int) (Convert.ToDouble(drow[col]) * (rand.Next(50) / 100.0f + 0.8));// rand.Next(100);
					//										else
					//									drow[col] = 2;
				}

				// System.Threading.Thread.Sleep(300);
				// Insert or remove a row
				if (insertRemoveCount == 0)
					return;

				if (toggleInsertRemove > 0 && (timerCount % insertRemoveModulus) == 0)
				{
					icount = ++icount % (toggleInsertRemove * 2);
					shouldInsert = icount < toggleInsertRemove;

					if (shouldInsert)
					{
						for (int ri = 0; ri < insertRemoveCount; ri++)
						{
							int recNum = 5;// rand.Next(table.Rows.Count - 1);

							int next = rand.Next(100);
							object[] row = new object[]{"H"+ti.ToString("00000"),next+1,next+2,
														   next+3,next+4,next+5,next+6,
														   next+7,next+8,next+9,next+10,
														   next+11,next+12,next+13,next+14,
														   next+15,next+16,next+17,
														   next+18,next+19,next+20};
							ti++;
							DataRow drow = table.NewRow();
							drow.ItemArray = row;
							table.Rows.InsertAt(drow, recNum);
							//table.Rows.Add(drow);

						}
					}
					else
					{
						for (int ri = 0; ri < insertRemoveCount; ri++)
						{
							int recNum = 5; //rand.Next(m_set.Count - 1);
							int rowNum = recNum + 1;

							// Underlying data structure (this could be a datatable or whatever structure
							// you use behind a virtual grid).

							if (table.Rows.Count > 10)
								table.Rows.RemoveAt(recNum);
						}
					}
				}

			}
			finally
			{
			}
		}

		Color[] colors = new Color[] {
										 
										 Color.FromArgb( 0x85, 0xbf, 0x75 ),
										 //Color.FromArgb( 0xde, 0x64, 0x13 ),
										 Color.FromArgb( 0xb4, 0xe7, 0xf2 ),
										 Color.FromArgb( 0xff, 0xbf, 0x34 ),
										 Color.FromArgb( 0x82, 0x2e, 0x1b ),
										 Color.FromArgb( 0x3a, 0x86, 0x7e ),
		};

		private void trackBarTimer_Scroll(object sender, System.EventArgs e)
		{
			if (this.trackBarTimer.Value == 0)
			{
				timer.Enabled = false;
				this.labelTimerInterval.Text = String.Format("Timer disabled.");
			}
			else
			{
				timer.Interval = 1000/(this.trackBarTimer.Value*trackBarTimer.Value);
				timer.Enabled = true;
				this.labelTimerInterval.Text = String.Format("Every {0} milliseconds.", timer.Interval);
			}
		}

		#region Update dialog controls with changes to grid
		private void gridGroupingControl1_PropertyChanged(object sender, DescriptorPropertyChangedEventArgs e)
		{
			UpdateDialog();
		}

		private void UpdateDialog()
		{
			if (isUIChanged)
				return;

			try
			{
				isPropertyChanged = true;

				GridTableDescriptor td = this.gridGroupingControl1.TableDescriptor;

				this.checkBoxFilter.Checked = td.RecordFilters.Count > 0;
				if (td.RecordFilters.Count > 0)
					this.textBoxFilter.Text  = td.RecordFilters[0].Expression;

				this.checkBoxGrouping.Checked = td.GroupedColumns.Count > 0;
				this.checkBoxSorting.Checked = td.SortedColumns.Count > 0;

				this.trackBarBlinkFrequency.Value = this.gridGroupingControl1.BlinkTime/100;
				if (this.gridGroupingControl1.BlinkTime == 0)
					this.labelBlinkTime.Text = String.Format("Disabled.");
				else
					this.labelBlinkTime.Text = String.Format("{0} milliseconds.", gridGroupingControl1.BlinkTime);

                if (timer != null)
                {
                    if (timer.Enabled)
                        trackBarTimer.Value = (int)Math.Sqrt(1000 / timer.Interval);
                    else
                        trackBarTimer.Value = 0;

                    trackBarTimer.TickFrequency = 1;
                    this.labelTimerInterval.Text = String.Format("Every {0} milliseconds.", timer.Interval);
                }
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
			}
			finally
			{
				isPropertyChanged = false;
			}
		}

		bool isUIChanged = false;
		bool isPropertyChanged = false;
		#endregion
		#region Filter
		private void checkBoxFilter_CheckedChanged(object sender, System.EventArgs e)
		{
			ApplyFilter();
		}

		private void buttonApplyFilter_Click(object sender, System.EventArgs e)
		{
			if (!checkBoxFilter.Checked)
				checkBoxFilter.Checked = true;
			else
				ApplyFilter();
		}

		private void ApplyFilter()
		{
			if (isPropertyChanged)
				return;

			isUIChanged = true;

			GridTableDescriptor td = this.gridGroupingControl1.TableDescriptor;
			try
			{
				if (this.checkBoxFilter.Checked)
				{
					td.RecordFilters.Clear();
					td.RecordFilters.Add(this.textBoxFilter.Text);
                }
				else
				{
					td.RecordFilters.Clear();
                }
                this.gridGroupingControl1.Refresh();
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
			}
			finally
			{
				isUIChanged = false;
			}
		}
		#endregion
		#region Grouping
		private void checkBoxGrouping_CheckedChanged(object sender, System.EventArgs e)
		{
			if (isPropertyChanged)
				return;

			isUIChanged = true;

			GridTableDescriptor td = this.gridGroupingControl1.TableDescriptor;
			try
			{
				if (this.checkBoxGrouping.Checked)
				{
					td.GroupedColumns.Clear();
					td.GroupedColumns.Add("1");
					this.gridGroupingControl1.Table.ExpandAllGroups();
                }
				else
				{
					td.GroupedColumns.Clear();
				}
				this.gridGroupingControl1.Refresh();
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
			}
			finally
			{
				isUIChanged = false;
			}
		}
		#endregion
		#region BlinkFrequency
		private void trackBarBlinkFrequency_Scroll(object sender, System.EventArgs e)
		{
			if (isPropertyChanged)
				return;

			isUIChanged = true;

			GridTableDescriptor td = this.gridGroupingControl1.TableDescriptor;
			try
			{
				this.gridGroupingControl1.BlinkTime = this.trackBarBlinkFrequency.Value*100;
				if (this.gridGroupingControl1.BlinkTime == 0)
					this.labelBlinkTime.Text = String.Format("Disabled.");
				else
					this.labelBlinkTime.Text = String.Format("{0} milliseconds.", gridGroupingControl1.BlinkTime);
				this.gridGroupingControl1.Refresh();
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
			}
			finally
			{
				isUIChanged = false;
			}
		}
		#endregion
		#region Sorting
		private void checkBoxSorting_CheckedChanged(object sender, System.EventArgs e)
		{
			if (isPropertyChanged)
				return;

			isUIChanged = true;

			GridTableDescriptor td = this.gridGroupingControl1.TableDescriptor;
			try
			{
				if (this.checkBoxSorting.Checked)
				{
					td.SortedColumns.Clear();
					td.SortedColumns.Add("1");
					td.SortedColumns.Add("2");
                }
				else
				{
					td.SortedColumns.Clear();
                }
				this.gridGroupingControl1.Refresh();
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
			}
			finally
			{
				isUIChanged = false;
			}

		}
		#endregion
		#region Coloring
		private void checkBoxColor_CheckedChanged(object sender, System.EventArgs e)
		{
			isUIChanged = true;

			GridTableDescriptor td = this.gridGroupingControl1.TableDescriptor;
			try
			{
				if (this.checkBoxColor.Checked)
				{
					// Callback for dynamically coloring cells
					gridGroupingControl1.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(gridGroupingControl1_QueryCellStyleInfo);

					// The color of these cells depends on value of cell "1". If engines ListChanged handler
					// detects a change to column "1" it should also automatically repaint the dependant columns
					for (int i = 2; i <= 20; i++)
						gridGroupingControl1.TableDescriptor.Fields[i.ToString()].ReferencedFields = "1";

				}
				else
				{
					gridGroupingControl1.QueryCellStyleInfo -= new GridTableCellStyleInfoEventHandler(gridGroupingControl1_QueryCellStyleInfo);
					gridGroupingControl1.TableDescriptor.Fields.LoadDefault();
                }
				this.gridGroupingControl1.Refresh();
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
			}
			finally
			{
				isUIChanged = false;
			}
		}
		#endregion


        protected override void OnLoad(EventArgs e)
        {
            new GridGroupingControlSchemaBrowser(this.gridGroupingControl1).Show();
            base.OnLoad(e);

            // Timer
            SetupTimer();
        }

    }
}

