Imports System
Imports System.Drawing
Imports System.Collections
Imports System.Diagnostics
Imports System.ComponentModel
Imports System.Data
Imports System.Text

Imports Syncfusion.Grouping
Imports Syncfusion.Windows.Forms.Grid.Grouping

'Namespace Syncfusion.Grouping.ManualSummaries

Public Class ManualTotalSummaryEngineFactory
    Inherits GridEngineFactoryBase

    ' Add these static ctor to your form:
    '
    '        static Form1()
    '        {
    '            // Enable ManualTotalSummaryEngineFactory with optimized ListChanged handler. Must
    '            // be set before InitializeComponent in Form ctor.
    '            GridEngineFactory.Factory = new ManualTotalSummaryEngineFactory();
    '        }
    '        

    Public Overrides Function CreateEngine() As GridEngine
        Return New ManualTotalSummaryEngine
    End Function 'CreateEngine
End Class 'ManualTotalSummaryEngineFactory


Public Class ManualTotalSummaryEngine
    Inherits GridEngine

    Public Sub New()
    End Sub 'New


    Public Overloads Overrides Function CreateTable(ByVal tableDescriptor As TableDescriptor, ByVal parentRelationTable As Table) As Table
        Return New ManualTotalSummaryTable(tableDescriptor, parentRelationTable)
    End Function 'CreateTable


    Public Overrides Function CreateGroup(ByVal parent As Section) As Group
        Return New ManualTotalSummaryGroup(parent)
    End Function 'CreateGroup


    Public Overrides Function CreateChildTable(ByVal parent As Element) As ChildTable
        Return New ManualTotalSummaryChildTable(parent)
    End Function 'CreateChildTable
End Class 'ManualTotalSummaryEngine

'/ <summary>
'/ Grouping child table with support for manual total summaries.
'/ </summary>

Public Class ManualTotalSummaryChildTable
    Inherits GridChildTable
    Implements IManualTotalSummaryArraySource
    Private ManualTotalSummaryArray() As ManualTotalSummary


    Public Sub New(ByVal parent As Element)
        MyBase.New(parent)
        ManualTotalSummaryArray = New ManualTotalSummary(parent.ParentTableDescriptor.Fields.Count) {}

        Dim td As ManualTotalSummaryTable = IIf(TypeOf parent Is Table, parent, parent.ParentTable)
        If Not (td Is Nothing) Then
            Dim field As String
            For Each field In td.TotalSummaries
                Dim obj As ManualTotalSummary = New ManualTotalSummary(Me, field)
            Next field
        End If
    End Sub 'New

#Region "IManualTotalSummaryArraySource Members"


    Public Function GetManualTotalSummaryArray() As ManualTotalSummary() Implements IManualTotalSummaryArraySource.GetManualTotalSummaryArray
        Return ManualTotalSummaryArray
    End Function 'GetManualTotalSummaryArray

#End Region
End Class 'ManualTotalSummaryChildTable

'/ <summary>
'/ Grouping group with support for manual total summaries.
'/ </summary>

Public Class ManualTotalSummaryGroup
    Inherits GridGroup
    Implements IManualTotalSummaryArraySource ', IGridGroupOptionsSource
    Private ManualTotalSummaryArray() As ManualTotalSummary


    Public Sub New(ByVal parent As Section)
        MyBase.New(parent)
        ManualTotalSummaryArray = New ManualTotalSummary(parent.ParentTableDescriptor.Fields.Count) {}

        Dim td As ManualTotalSummaryTable = parent.ParentTable '

        If Not (td Is Nothing) Then
            Dim field As String
            For Each field In td.TotalSummaries
                Dim obj As ManualTotalSummary = New ManualTotalSummary(Me, field)
            Next field
        End If
    End Sub 'New


    '`    Function GetParentGroupOptionsSource() As IGridGroupOptionsSource Implements IGridGroupOptionsSource.GetParentGroupOptionsSource
    '        ' this method is overriden to avoid looking up Appearance from GridColumnDescriptor
    '        ' which costs a few ticks performance.
    '        Return CType(Me.ParentTableDescriptor, IGridGroupOptionsSource)
    '    End Function 'IGridGroupOptionsSource.GetParentGroupOptionsSource

#Region "IManualTotalSummaryArraySource Members"


    Public Function GetManualTotalSummaryArray() As ManualTotalSummary() Implements IManualTotalSummaryArraySource.GetManualTotalSummaryArray
        Return ManualTotalSummaryArray
    End Function 'GetManualTotalSummaryArray

#End Region
End Class 'ManualTotalSummaryGroup


'/ <summary>
'/ Table with ManualTotalSummaries.
'/ </summary>

Public Class ManualTotalSummaryTable
    Inherits GridTable

    Public Sub New(ByVal tableDescriptor As TableDescriptor, ByVal parentRelationTable As Table)
        MyBase.New(CType(tableDescriptor, GridTableDescriptor), CType(parentRelationTable, GridTable))
    End Sub 'New

#Region "TotalSummaries Support"
    Private _totalSummaries As New ArrayList

    '/ <exclude/>
    '/ <summary>
    '/ TODO: Move this to TableDescriptor.
    '/ </summary>

    Public Property TotalSummaries() As ArrayList
        Get
            Return Me._totalSummaries
        End Get
        Set(ByVal Value As ArrayList)
            Me._totalSummaries = Value
        End Set
    End Property


    Protected Overrides Sub OnPrepareRemoving(ByVal row As Object)
        ' Save values for all fields where we need to be able to access the
        ' old value (e.g. Delta for TotalSummaries).
        Dim td As TableDescriptor = TableDescriptor
        Dim tsa As IManualTotalSummaryArraySource = Me.TopLevelGroup '
        If Not (tsa Is Nothing) Then
            Dim name As String
            For Each name In Me.TotalSummaries
                Dim fd As FieldDescriptor = td.Fields(name)
                If fd.IsPropertyField() Then
                    Dim pd As PropertyDescriptor = fd.GetPropertyDescriptor()
                    Dim value As Object = GetValue(row, pd)

                    Dim ci As New ChangedFieldInfo(td, pd.Name, value, Nothing)
                    Me.AddChangedField(ci)
                End If
            Next name
        End If

        MyBase.OnPrepareRemoving(row)
    End Sub 'OnPrepareRemoving


    Protected Overrides Sub OnPrepareItemAdded(ByVal row As Object)
        ' Get new values for which delta information is needed
        Dim tsa As IManualTotalSummaryArraySource = Me.TopLevelGroup '
        If Not (tsa Is Nothing) Then
            Dim td As TableDescriptor = TableDescriptor

            Dim name As String
            For Each name In Me.TotalSummaries
                Dim fd As FieldDescriptor = td.Fields(name)
                If fd.IsPropertyField() Then
                    Dim pd As PropertyDescriptor = fd.GetPropertyDescriptor()
                    Dim value As Object = GetValue(row, pd)

                    Dim ci As New ChangedFieldInfo(td, pd.Name, Nothing, value)
                    Me.AddChangedField(ci)
                End If
            Next name
        End If

        MyBase.OnPrepareItemAdded(row)
    End Sub 'OnPrepareItemAdded


    Protected Overrides Sub OnRecordChanged(ByVal r As Element, ByVal isObsoleteRecord As Boolean, ByVal isAddedRecord As Boolean)
        Dim td As TableDescriptor = TableDescriptor
        Dim g As Group = r.ParentGroup
        While TypeOf g Is IManualTotalSummaryArraySource
            OnGroupSummaryInvalidated(New GroupEventArgs(g))

            Dim tsa As IManualTotalSummaryArraySource = g '
            Dim ci As ChangedFieldInfo
            For Each ci In Me.ChangedFieldsArray
                Dim mt As ManualTotalSummary = tsa.GetManualTotalSummaryArray()(ci.FieldIndex)
                If Not (mt Is Nothing) Then
                    mt.ApplyDelta(r, isObsoleteRecord, isAddedRecord, ci)
                End If
            Next ci
            g = g.ParentGroup
        End While
    End Sub 'OnRecordChanged ' Fix ManualTotalSummary of parent groups.
#End Region
End Class 'ManualTotalSummaryTable




'/ <summary>
'/ Implement this interface in GridGroup and GridChildTable so that others have access to ManualTotalSummary array
'/ </summary>

Public Interface IManualTotalSummaryArraySource
    Function GetManualTotalSummaryArray() As ManualTotalSummary()
End Interface 'IManualTotalSummaryArraySource

'/ <summary>
'/ This is a manual summary class which can be updated immediately using the difference between old and
'/ new value in a ListChanged event. For each entry in ManualTotalSummaryTable.TotalSummaries a ManualTotalSummary
'/ will be created. In gridGroupingControl1_QueryCellStyleInfo the ManualTotalSummary.Total value will 
'/ be retrieved and displayed in a summary or caption cell.
'/ </summary>

Public Class ManualTotalSummary
    Private _total As Double
    Private _dirty As Boolean = True
    Private _group As Group
    Private _fieldIndex As Integer = -1


    Public Sub New(ByVal g As Group, ByVal field As String)
        MyClass.New(g, g.ParentTableDescriptor.Fields(field))
    End Sub 'New


    Public Sub New(ByVal g As Group, ByVal field As FieldDescriptor)
        Me.Field = field
        Me.Group = g
    End Sub 'New


    Public Property FieldIndex() As Integer
        Get
            If _FieldIndex = -1 Then
                _FieldIndex = field.Collection.IndexOf(Field)
            End If
            Return Me._fieldIndex
        End Get
        Set(ByVal Value As Integer)
            _fieldIndex = Value
        End Set
    End Property



    Public Property Group() As Group
        Get
            Return Me._group
        End Get
        Set(ByVal Value As Group)
            Me._group = Value
            Dim tsa As IManualTotalSummaryArraySource = _group '
            tsa.GetManualTotalSummaryArray()(FieldIndex) = Me
        End Set
    End Property

    Private _field As FieldDescriptor


    Public Property Field() As FieldDescriptor
        Get
            Return Me._field
        End Get
        Set(ByVal Value As FieldDescriptor)
            Me._field = Value
        End Set
    End Property


    Public Property Dirty() As Boolean
        Get
            Return Me._dirty
        End Get
        Set(ByVal Value As Boolean)
            Me._dirty = Value
        End Set
    End Property


    Public Property Total() As Double
        Get
            If _dirty Then
                CalculateTotal()
                Me._dirty = False
            End If


            Return Me._total
        End Get
        Set(ByVal Value As Double)
            Me._total = Value
        End Set
    End Property


    Sub CalculateTotal()
        _total = 0

        If TypeOf group.Details Is RecordsDetails Then
            Dim r As Record
            For Each r In group.Records
                Dim obj As Object = r.GetValue(field)
                If Not (obj Is Nothing) AndAlso Not TypeOf obj Is DBNull Then
                    Dim d As Double = Convert.ToDouble(obj)
                    _total += d
                End If
            Next r
        Else
            Dim g As Group
            For Each g In group.Groups
                Dim tsa As IManualTotalSummaryArraySource = g '
                Dim mt As ManualTotalSummary = tsa.GetManualTotalSummaryArray()(Me.FieldIndex)
                If mt Is Nothing Then
                    mt = New ManualTotalSummary(g, field)
                End If
                Dim d As Double = mt.Total
                _total += d
            Next g
        End If
    End Sub 'CalculateTotal


    Public Sub ApplyDelta(ByVal r As Element, ByVal isObsoleteRecord As Boolean, ByVal isAddedRecord As Boolean, ByVal ci As ChangedFieldInfo)
        If Dirty Then
            Return
        End If
        Dim mt As ManualTotalSummary = Me

        If isObsoleteRecord Then
            If Not (ci.OldValue Is Nothing) AndAlso Not TypeOf ci.OldValue Is DBNull Then
                mt.Total -= Convert.ToDouble(ci.OldValue)
            End If
        ElseIf isAddedRecord Then
            If Not (ci.NewValue Is Nothing) AndAlso Not TypeOf ci.NewValue Is DBNull Then
                mt.Total += Convert.ToDouble(ci.NewValue)
            End If
        Else
            mt.Total += ci.Delta
        End If
    End Sub 'ApplyDelta 
End Class 'ManualTotalSummary
'End Namespace 'Syncfusion.Grouping.ManualSummaries
