using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.Windows.Forms.Grid;
using System.Data;
using System.Drawing;
using Syncfusion.Grouping;

using Syncfusion.Windows.Forms.Grid.Grouping;
using System.Windows.Forms;
using System.ComponentModel;

#if !SyncfusionFramework2_0
namespace System.Collections.Generic
{
}
#endif

namespace MarketSimulation
{
    class GridC : GridGroupingControl
    {
        public void SetStyle()
        {
            this.BlinkTime = 700;
//            this.UpdateDisplayFrequency = 0; // manually call Update for grid.
//            this.UpdateDisplayFrequency = 1; // paint after each change.
            this.UpdateDisplayFrequency = 50; // paint every 50 ms. (but scroll window will force immediate paint ...)
			this.SortPositionChangedBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate;
			this.InsertRemoveBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate;
			this.BindToCurrencyManager = false;
			this.UseDefaultsForFasterDrawing = true;

            ShowNavigationBar = false;
            ShowGroupDropArea = true;
            TopLevelGroupOptions.ShowCaption = false;

            //TableDescriptor.ChildGroupOptions.ShowCaptionSummaryCells = true;
            TableDescriptor.ChildGroupOptions.ShowSummaries = false;

            // Fonts
            this.Font = new Font("Tahoma", (float)9.0, FontStyle.Regular);
            GridGroupDropArea.Font = this.Font;

            // Control Back Color
            BackColor = Color.FromKnownColor(KnownColor.Control);

            // General Cell Colour
            Appearance.AnyRecordFieldCell.BackColor = Color.FromArgb(237, 240, 246);
            Appearance.AnyRecordFieldCell.TextColor = Color.FromArgb(57, 73, 122);            

            // Column Headers Cells
            Appearance.ColumnHeaderCell.Themed = false;
            Appearance.ColumnHeaderCell.BackColor = Color.FromArgb(0, 21, 84);
            Appearance.ColumnHeaderCell.TextColor = Color.FromArgb(238, 122, 3);
            Appearance.ColumnHeaderCell.Font.Bold = true;

            // Caption Cells
            Appearance.GroupCaptionCell.BackColor = Appearance.AnyRecordFieldCell.BackColor;
            Appearance.GroupCaptionCell.TextColor = Appearance.AnyRecordFieldCell.TextColor;
            Appearance.GroupCaptionCell.Borders.Top = new GridBorder(GridBorderStyle.Standard);
            Appearance.GroupCaptionCell.CellType = "Static";
            
            // General Table Options
            TableOptions.CaptionRowHeight = TableOptions.RecordRowHeight;
            TableOptions.IndentWidth = 16;
            TableOptions.ListBoxSelectionMode = System.Windows.Forms.SelectionMode.One;
            TableOptions.ListBoxSelectionCurrentCellOptions = Syncfusion.Windows.Forms.Grid.Grouping.GridListBoxSelectionCurrentCellOptions.HideCurrentCell;
            TableOptions.ListBoxSelectionColorOptions = Syncfusion.Windows.Forms.Grid.Grouping.GridListBoxSelectionColorOptions.ApplySelectionColor;
            TableOptions.ShowRowHeader = false;            

            // Highlighted Cells - doesn't seem to work
            TableOptions.SelectionBackColor = Color.Khaki;
            TableOptions.SelectionTextColor = Color.Black;

            // Can't add rows
            ChildGroupOptions.ShowAddNewRecordBeforeDetails = false;
            TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
            NestedTableGroupOptions.ShowAddNewRecordBeforeDetails = false;

			Engine.QueryAddColumn += new GridQueryAddColumnEventHandler(Engine_QueryAddColumn);
        }



        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }

        #region SetBoundList

        public static DataTable CreateDataTable(OrderAgreementListC ocList)
        {
            DataTable table = new DataTable();

            if (ocList == null)
                return table;

            OrderAgreementC record;

            if (ocList.Count == 0)
            {
                record = new OrderAgreementC();
            }
            else
            {
                record = ocList[0];
            }

            TypeSystemC.FidEnum[] recordKeys = new TypeSystemC.FidEnum[record.Count];
            record.Keys.CopyTo(recordKeys, 0);

#if SyncfusionFramework2_0
            foreach (TypeSystemC.FidEnum fid in recordKeys)
            {
#else
            for (int n = recordKeys.Length-1; n >= 0; n--)
            {
                TypeSystemC.FidEnum fid = recordKeys[n];
#endif
            //foreach (TypeSystemC.FidEnum fid in record.Keys)
            //{
                string name = TypeSystemC.FidName(fid);
                Type type = TypeSystemC.FidType(fid);

                DataColumn col = new DataColumn(name, type);
                table.Columns.Add(col);
            }

            foreach (OrderAgreementC oc in ocList)
            {   
                object[] row = new object[oc.Count];

#if SyncfusionFramework2_0
                int i = 0;
                foreach (object o in oc.Values)
                {
                    row[i++] = o;
                }
#else
                int i = oc.Count-1;
                foreach (object o in oc.Values)
                {
                    row[i--] = o;
                }
#endif

                table.Rows.Add(row);
            }

            return table;
        }

		#endregion

		private void Engine_QueryAddColumn(object sender, GridQueryAddColumnEventArgs e)
		{
			e.GridColumn.AllowBlink = true;
		}
	}
}

