using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Grouping;

namespace MarketSimulation
{
#if SyncfusionFramework2_0
    class RecordList : List<Record>
    {
    }

    class RecordListDictionary : Dictionary<string, RecordList>
    {
    }
#else
    class RecordList : ArrayList
    {
        public new Record this[int index]
        {
            get
            {
                return (Record) base[index];
            }
            set
            {
                base[index] = value;
            }
        }
    }

    class RecordListDictionary : Hashtable
    {
        public RecordList this[string key]
        {
            get
            {
                return (RecordList) base[key];
            }
            set
            {
                base[key] = value;
            }
        }
    }
#endif

    internal class OrderAgreementQueuerC : System.Windows.Forms.Form
    {
        private QuoteManagerC.QuotesUpdatedDelegate quotesUpdateDelegate;

        // Added these to try and improve performance
        private FieldDescriptor symbolColumn;
        private FieldDescriptor openColumn;
        private FieldDescriptor bidColumn;
        private FieldDescriptor askColumn;
        private FieldDescriptor lastColumn;
        private GridC grid;

        private RecordListDictionary symbolRecordMap = new RecordListDictionary();

        public OrderAgreementQueuerC()
        {
            InitializeComponent();
			grid.SetStyle();
			LoadData();
			System.Drawing.Icon ico = new System.Drawing.Icon(Syncfusion.Windows.Forms.WinFormsUtils.FindFile("Icon", "sfgrid.ico"));
			this.Icon = ico;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            BeginInvoke(new MethodInvoker(subscribeQuotes));
        }

        void subscribeQuotes()
        {
            Application.DoEvents();
            QuoteManagerC.Manager.QuotesUpdate += new QuoteManagerC.QuotesUpdatedDelegate(Manager_QuotesUpdate);
        }

        void Manager_QuotesUpdate(QuoteC[] quoteList)
        {
            // Quote updates come in on a separate thread, so call Control.Invoke()
            try
            {
                if (!IsDisposed)
                    Invoke(quotesUpdateDelegate, new object[] { quoteList });
            }
            catch (Exception)
            { }
        }

        void QuotesUpdate(QuoteC[] quoteList)
        {
			if (grid.IsDisposed)
				return;

            foreach (QuoteC quote in quoteList)
            {
                if (symbolRecordMap.ContainsKey(quote.Symbol))
                {
                    foreach (Record record in symbolRecordMap[quote.Symbol])
                    {
                        DataRow data = (DataRow) record.GetData();
                        data.BeginEdit();
                        data[openColumn.MappingName] = quote.Open;
                        data[bidColumn.MappingName] = quote.Bid;
                        data[askColumn.MappingName] = quote.Ask;
                        data[lastColumn.MappingName] = quote.Last;
                        data.EndEdit();
                    }
                }
            }

            this.grid.Update(); // calls PaintUpdateRecordFields.
        }

        protected void LoadData()
        {
            // In this prototype, the orders will not change... but the market data will
            OrderAgreementListC records = new OrderAgreementListC();

            for (int i = 0; i < Program.NumberOfRows; i++)
            {
                OrderAgreementC orderAgreement = OrderAgreementC.GenerateRandom();
                records.Add(orderAgreement);
            }

            DataTable table = GridC.CreateDataTable(records);

            grid.BindToCurrencyManager = false;
            grid.AllowSwapDataViewWithDataTableList = true;
            grid.DataSource = table;

            symbolColumn = grid.TableDescriptor.Fields["Symbol"];
            openColumn = grid.TableDescriptor.Fields["Opn"];
            bidColumn = grid.TableDescriptor.Fields["Bid"];
            askColumn = grid.TableDescriptor.Fields["Ask"];
            lastColumn = grid.TableDescriptor.Fields["Last"];

            // Cache symbol -> records map for faster market data updates
            int count = grid.Table.Records.Count;
            for (int n = 0; n < count; n++)
            {
                Record record = grid.Table.Records[n];
                string symbol = (string)record.GetValue(symbolColumn);
                if (symbol == null)
                    continue;

                RecordList recordList;                
                if (symbolRecordMap.ContainsKey(symbol))
                {
                    recordList = symbolRecordMap[symbol];
                }
                else
                {
                    recordList = new RecordList();
                    symbolRecordMap.Add(symbol, recordList);
                }
                recordList.Add(record);
            }

            quotesUpdateDelegate = new QuoteManagerC.QuotesUpdatedDelegate(QuotesUpdate);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        protected void InitializeComponent()
        {
            this.grid = new GridC();
            ((System.ComponentModel.ISupportInitialize)(this.grid)).BeginInit();
            this.SuspendLayout();
            // 
            // grid
            // 
            this.grid.BackColor = System.Drawing.SystemColors.Window;
            this.grid.Dock = System.Windows.Forms.DockStyle.Fill;
            this.grid.Location = new System.Drawing.Point(0, 0);
            this.grid.Name = "grid";
            this.grid.TabIndex = 0;
            this.grid.Text = "gridGroupingControl1";
            // 
            // OrderAgreementQueuerC
            // 
            this.ClientSize = new System.Drawing.Size(800, 450);
            this.Controls.Add(this.grid);
            this.Name = "OrderAgreementQueuerC";
            this.Text = "Order Agreements";
            ((System.ComponentModel.ISupportInitialize)(this.grid)).EndInit();
            this.ResumeLayout(false);

        }

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                QuoteManagerC.Manager.QuotesUpdate -= new QuoteManagerC.QuotesUpdatedDelegate(Manager_QuotesUpdate);
                QuoteManagerC.Manager.Dispose();
                if (components != null)
                    components.Dispose();
            }
            base.Dispose(disposing);
        }

        #endregion
    }
}