Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports Syncfusion.Grouping

Namespace MarketSimulation
#If SyncfusionFramework2_0 Then
	Friend Class RecordList : Inherits List([Of] Record)
	End Class

	Friend Class RecordListDictionary : Inherits Dictionary([Of] String : Implements RecordList)
	End Class
#Else
	Friend Class RecordList : Inherits ArrayList
		Public Shadows Default Property Item(ByVal index As Integer) As Record
			Get
				Return CType(MyBase.Item(index), Record)
			End Get
			Set
				MyBase.Item(index) = Value
			End Set
		End Property
	End Class

	Friend Class RecordListDictionary : Inherits Hashtable
        Default Public Overloads Property Item(ByVal key As String) As RecordList
            Get
                Return CType(MyBase.Item(key), RecordList)
            End Get
            Set(ByVal Value As RecordList)
                MyBase.Item(key) = Value
            End Set
        End Property
    End Class
#End If

	Friend Class OrderAgreementQueuerC : Inherits System.Windows.Forms.Form
		Private quotesUpdateDelegate As QuoteManagerC.QuotesUpdatedDelegate

		' Added these to try and improve performance
		Private symbolColumn As FieldDescriptor
		Private openColumn As FieldDescriptor
		Private bidColumn As FieldDescriptor
		Private askColumn As FieldDescriptor
		Private lastColumn As FieldDescriptor
		Private grid As GridC

		Private symbolRecordMap As RecordListDictionary = New RecordListDictionary()

		Public Sub New()
			InitializeComponent()
            grid.SetStyles()
            LoadData()
            Dim ico As System.Drawing.Icon = New System.Drawing.Icon(Syncfusion.Windows.Forms.WinFormsUtils.FindFile("Icon", "sfgrid.ico"))
			Me.Icon = ico
		End Sub

		Protected Overrides Sub OnLoad(ByVal e As EventArgs)
			MyBase.OnLoad(e)

			BeginInvoke(New MethodInvoker(AddressOf subscribeQuotes))
		End Sub

		Private Sub subscribeQuotes()
			Application.DoEvents()
			AddHandler QuoteManagerC.Manager.QuotesUpdate, AddressOf Manager_QuotesUpdate
		End Sub

		Private Sub Manager_QuotesUpdate(ByVal quoteList As QuoteC())
			' Quote updates come in on a separate thread, so call Control.Invoke()
			Try
				If (Not IsDisposed) Then
					Invoke(quotesUpdateDelegate, New Object() { quoteList })
				End If
			Catch e1 As Exception
			End Try
		End Sub

		Private Sub QuotesUpdate(ByVal quoteList As QuoteC())
			If grid.IsDisposed Then
				Return
			End If
            Dim quote As QuoteC
            For Each quote In quoteList
                If symbolRecordMap.ContainsKey(quote.Symbol) Then
                    Dim rec As Record
                    For Each rec In symbolRecordMap(quote.Symbol)
                        Dim data As DataRow = CType(rec.GetData(), DataRow)
                        data.BeginEdit()
                        data(openColumn.MappingName) = quote.Open
                        data(bidColumn.MappingName) = quote.Bid
                        data(askColumn.MappingName) = quote.Ask
                        data(lastColumn.MappingName) = quote.Last
                        data.EndEdit()
                    Next rec
                End If
            Next quote

            Me.grid.Update() ' calls PaintUpdateRecordFields.
		End Sub

		Protected Sub LoadData()
			' In this prototype, the orders will not change... but the market data will
			Dim records As OrderAgreementListC = New OrderAgreementListC()

			Dim i As Integer = 0
            Do While i < Program.NumberOfRows
                Dim orderAgreement As OrderAgreementC = OrderAgreementC.GenerateRandom()
                records.Add(orderAgreement)
                i += 1
            Loop

            Dim table As DataTable = GridC.CreateDataTable(records)

            grid.BindToCurrencyManager = False
            grid.AllowSwapDataViewWithDataTableList = True
            grid.DataSource = table

            symbolColumn = grid.TableDescriptor.Fields("Symbol")
            openColumn = grid.TableDescriptor.Fields("Opn")
            bidColumn = grid.TableDescriptor.Fields("Bid")
            askColumn = grid.TableDescriptor.Fields("Ask")
            lastColumn = grid.TableDescriptor.Fields("Last")

            ' Cache symbol -> records map for faster market data updates
            Dim count As Integer = grid.Table.Records.Count
            Dim n As Integer = 0
            Do While n < count
                Dim record As record = grid.Table.Records(n)
                Dim symbol As String = CStr(record.GetValue(symbolColumn))
                If symbol Is Nothing Then
                    GoTo Continue1
                End If

                Dim recordList As recordList
                If symbolRecordMap.ContainsKey(symbol) Then
                    recordList = symbolRecordMap(symbol)
                Else
                    recordList = New recordList
                    symbolRecordMap.Add(symbol, recordList)
                End If
                recordList.Add(record)
                n += 1
Continue1:
            Loop

            quotesUpdateDelegate = New QuoteManagerC.QuotesUpdatedDelegate(AddressOf QuotesUpdate)
		End Sub

		#Region "Windows Form Designer generated code"

		''' <summary>
		''' Required method for Designer support - do not modify
		''' the contents of this method with the code editor.
		''' </summary>
		Protected Sub InitializeComponent()
			Me.grid = New GridC()
			CType(Me.grid, System.ComponentModel.ISupportInitialize).BeginInit()
			Me.SuspendLayout()
			' 
			' grid
			' 
			Me.grid.BackColor = System.Drawing.SystemColors.Window
			Me.grid.Dock = System.Windows.Forms.DockStyle.Fill
			Me.grid.Location = New System.Drawing.Point(0, 0)
			Me.grid.Name = "grid"
			Me.grid.TabIndex = 0
			Me.grid.Text = "gridGroupingControl1"
			' 
			' OrderAgreementQueuerC
			' 
			Me.ClientSize = New System.Drawing.Size(800, 450)
			Me.Controls.Add(Me.grid)
			Me.Name = "OrderAgreementQueuerC"
			Me.Text = "Order Agreements"
			CType(Me.grid, System.ComponentModel.ISupportInitialize).EndInit()
			Me.ResumeLayout(False)

		End Sub

		''' <summary>
		''' Required designer variable.
		''' </summary>
		Private components As System.ComponentModel.IContainer = Nothing

		''' <summary>
		''' Clean up any resources being used.
		''' </summary>
		''' <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			If disposing Then
				RemoveHandler QuoteManagerC.Manager.QuotesUpdate, AddressOf Manager_QuotesUpdate
				QuoteManagerC.Manager.Dispose()
				If Not components Is Nothing Then
					components.Dispose()
				End If
			End If
			MyBase.Dispose(disposing)
		End Sub

		#End Region
	End Class
End Namespace