#Region "Copyright Syncfusion Inc. 2001 - 2006"
'
'  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
'
'  Use of this code is subject to the terms of our license.
'  A copy of the current license can be obtained at any time by e-mailing
'  licensing@syncfusion.com. Any infringement will be prosecuted under
'  applicable laws. 
'
#End Region

Imports System
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Data

Imports Syncfusion.Drawing
Imports Syncfusion.Windows.Forms.Grid
Imports Syncfusion.Windows.Forms.Grid.Grouping
Imports Syncfusion.Grouping

'/ <summary>
'/ Summary description for Form1.
'/ </summary>

Public Class Form1
    Inherits System.Windows.Forms.Form '
    Private gridGroupingControl1 As Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl
    Private northwindDataSet1 As DataSet1
    Private sqlSelectCommand2 As System.Data.SqlClient.SqlCommand
    Private sqlInsertCommand2 As System.Data.SqlClient.SqlCommand
    Private sqlUpdateCommand2 As System.Data.SqlClient.SqlCommand
    Private sqlDeleteCommand2 As System.Data.SqlClient.SqlCommand
    Private sqlDataAdapterOrders As System.Data.SqlClient.SqlDataAdapter
    Private sqlConnection2 As System.Data.SqlClient.SqlConnection
    
    Private components As System.ComponentModel.Container = Nothing
    

    Sub OptimizeGridSettings(ByVal grid As GridGroupingControl)
        grid.CounterLogic = EngineCounters.YAmount
        grid.AllowedOptimizations = EngineOptimizations.DisableCounters Or EngineOptimizations.RecordsAsDisplayElements
        grid.UseDefaultsForFasterDrawing = True ' GDI interop drawing, FirstNRecords Summaries
        grid.InvalidateAllWhenListChanged = False ' only paint modified cells 
        grid.InsertRemoveBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate
        grid.UpdateDisplayFrequency = 50 ' update every number of ms (speciy 0 if Update should occur manually, 1 if immediately)
        'grid.TableControl.EnableDoubleBufferSurface(); // reduce flickering
        grid.BlinkTime = 700

        ' Instruct grouping control not to access datasource through
        ' CurrencyManager API and instead access list diretly. When set to
        ' false you wil howver loose support for built-in CurrencyManager 
        ' synchronization.
        grid.BindToCurrencyManager = False

        If GetType(Object).AssemblyQualifiedName.IndexOf("Version=1") = -1 Then
            ' .NET 2.0 gets very slow when subscribing to ListChanged event on the 
            ' DataTable.DefaultView. The following setting allows the engine to wrap 
            ' datatable with a Syncfusion.Collections.DataTableList to work around 
            ' this bottleneck.
            '
            ' .NET 1.0 and 1.1 are just fine with using DataTable.DefaultView
            '
            grid.AllowSwapDataViewWithDataTableList = True
        End If
    End Sub 'OptimizeGridSettings
    
    Public Sub New()
        '
        ' Required for Windows Form Designer support
        '
        InitializeComponent()
        
        '
        ' TODO: Add any constructor code after InitializeComponent call
        '
        InitializeDataSet()

        Me.OptimizeGridSettings(Me.gridGroupingControl1)

        Try
            Dim ico As Icon = New Icon(Syncfusion.Windows.Forms.WinFormsUtils.FindFile("Icon", "sfgrid.ico"))
            Me.Icon = ico
        Catch
        End Try
        Dim ordersDescriptor As GridTableDescriptor = Me.gridGroupingControl1.TableDescriptor

        ' Add Unbound 
        ordersDescriptor.UnboundFields.Add("ShipVia_CompanyName")

        ' Callback for conversion
        AddHandler ordersDescriptor.QueryValue, AddressOf ordersDescriptor_QueryValue

        ' Hide it.
        ordersDescriptor.VisibleColumns.Remove("ShipVia_CompanyName")

        ' Use it for grouping.
        ordersDescriptor.GroupedColumns.Add("ShipVia_CompanyName")

        ordersDescriptor.TopLevelGroupOptions.ShowCaptionSummaryCells = False

        ordersDescriptor.PrimaryKeyColumns.Clear()

        ' You can define a summary row and mark it hidden.
        ' In that summary row you can add a column and set it's mapping name (and DisplayColumn) to be Freight
        Dim summaryColumn1 As New GridSummaryColumnDescriptor("FreightAverage", SummaryType.DoubleAggregate, "Freight", "{Average:###.00}")
        Dim summaryRow1 As New GridSummaryRowDescriptor()
        summaryRow1.Name = "Caption"
        summaryRow1.Visible = False
        summaryRow1.SummaryColumns.Add(summaryColumn1)
        ordersDescriptor.SummaryRows.Add(summaryRow1)

        ' This is a second row, not marked hidden and therefore shown at the end of the group.
        Dim summaryColumn2 As New GridSummaryColumnDescriptor("FreightTotal", SummaryType.DoubleAggregate, "Freight", "{Average:###.00}")
        Dim summaryRow2 As New GridSummaryRowDescriptor()
        summaryRow2.Name = "Bottom"
        summaryRow2.Visible = True
        summaryRow2.SummaryColumns.Add(summaryColumn2)
        ordersDescriptor.SummaryRows.Add(summaryRow2)

        ' Here you define the summary row that should be used for displaying summaries in caption bar.
        ordersDescriptor.ChildGroupOptions.ShowCaptionSummaryCells = True
        ordersDescriptor.ChildGroupOptions.ShowSummaries = True
        ordersDescriptor.ChildGroupOptions.CaptionSummaryRow = "Caption"

        ' Let's you hide/show the second row in child groups.
        ordersDescriptor.ChildGroupOptions.ShowSummaries = False

        ' Move Freight column ahead 
        ' ordersDescriptor.VisibleColumns.LoadDefault();
        Dim count As Integer = ordersDescriptor.VisibleColumns.Count ' force populating VisibleColumns
        ordersDescriptor.VisibleColumns.Remove("Freight")
        ordersDescriptor.VisibleColumns.Insert(3, New GridVisibleColumnDescriptor("Freight"))


        ' Minor customization: Have Caption bar cover some cells.
        AddHandler Me.gridGroupingControl1.TableModel.QueryCoveredRange, AddressOf TableModel_QueryCoveredRange
        captionCoverCols = 2


        ' Workaround in 2.1.0.9 if SummaryCaptions should also be shown in TopLevelGroup
        'this.gridGroupingControl1.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(gridGroupingControl1_QueryCellStyleInfo);
        Me.gridGroupingControl1.Appearance.GroupCaptionCell.BackColor = Me.gridGroupingControl1.Appearance.RecordFieldCell.BackColor
        Me.gridGroupingControl1.Appearance.GroupCaptionCell.Borders.Top = New GridBorder(GridBorderStyle.Standard)
        Me.gridGroupingControl1.Appearance.GroupCaptionCell.CellType = "Static"

        Me.gridGroupingControl1.TableOptions.CaptionRowHeight = Me.gridGroupingControl1.TableOptions.RecordRowHeight

        Me.gridGroupingControl1.ChildGroupOptions.ShowAddNewRecordBeforeDetails = False

        Me.gridGroupingControl1.Table.CurrentRecord = Me.gridGroupingControl1.Table.Records(0)
    End Sub 'New


    Protected Overrides Sub OnLoad(ByVal e As EventArgs)
        MyBase.OnLoad(e)

        Dim startTimerButton As New Button
        startTimerButton.Bounds = New Rectangle(400, 5, 100, 20)
        startTimerButton.Text = "Start Timer"
        startTimerButton.Visible = True
        Me.Controls.Add(startTimerButton)
        AddHandler startTimerButton.Click, AddressOf startTimerButton_Click
        startTimerButton.BringToFront()
    End Sub 'OnLoad


    Sub startTimerButton_Click(ByVal sender As Object, ByVal e As EventArgs)
        ' Start timer that updates datasource 
        Dim t As New Timer
        t.Interval = 10
        AddHandler t.Tick, AddressOf t_Tick
        t.Start()
    End Sub 'startTimerButton_Click

    Private rnd As New Random


    Sub t_Tick(ByVal sender As Object, ByVal e As EventArgs)

        Dim n As Integer
        For n = 0 To 11
            Dim i As Integer = rnd.Next(Me.northwindDataSet1.Orders.Rows.Count)
            northwindDataSet1.Orders(i).Freight = northwindDataSet1.Orders(i).Freight + 3 - Math.Round(System.Convert.ToDecimal(rnd.NextDouble()) * 6)
        Next n
    End Sub 't_Tick

#Region "Initialize DataSet"

    Private xmlName As String = "Orders.xml"
    Private msdeAvailable As Boolean = False


    Sub InitializeDataSet()
        If msdeAvailable Then
            ' Initialize the data set
            Me.northwindDataSet1.BeginInit()

            ' Order is important here - first fill tables that are referenced with 
            ' foreign keys, then tables that have references to others.
            Me.sqlDataAdapterOrders.Fill(Me.northwindDataSet1)

            Me.northwindDataSet1.EndInit()

            Me.northwindDataSet1.WriteXml(xmlName)
        Else
            ' Read from a xml file instead. 
            If Not ReadXml(Me.northwindDataSet1, xmlName) Then
                ' If no xml found, try again with MSDE
                msdeAvailable = True
                InitializeDataSet()
            End If
        End If
    End Sub 'InitializeDataSet


    Function ReadXml(ByVal ds As DataSet, ByVal xmlFileName As String) As Boolean
        ' Check both in parent folder and Parent\Data folders.
        Dim xmlDataFileName As String = "Data\" + xmlFileName
        Dim n As Integer
        For n = 0 To 11
            If System.IO.File.Exists(xmlFileName) Then
                ds.ReadXml(xmlFileName)
                Return True
            End If
            If System.IO.File.Exists(xmlDataFileName) Then
                ds.ReadXml(xmlDataFileName)
                Return True
            End If
            xmlFileName = "..\" + xmlFileName
            xmlDataFileName = "..\" + xmlDataFileName
        Next n

        Return False
    End Function 'ReadXml

#End Region


    '/ <summary>
    '/ Clean up any resources being used.
    '/ </summary>
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub 'Dispose

#Region "Windows Form Designer generated code"

    '/ <summary>
    '/ Required method for Designer support - do not modify
    '/ the contents of this method with the code editor.
    '/ </summary>
    Private Sub InitializeComponent()
        Me.northwindDataSet1 = New DataSet1
        Me.gridGroupingControl1 = New Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl
        Me.sqlSelectCommand2 = New System.Data.SqlClient.SqlCommand
        Me.sqlInsertCommand2 = New System.Data.SqlClient.SqlCommand
        Me.sqlUpdateCommand2 = New System.Data.SqlClient.SqlCommand
        Me.sqlDeleteCommand2 = New System.Data.SqlClient.SqlCommand
        Me.sqlDataAdapterOrders = New System.Data.SqlClient.SqlDataAdapter
        Me.sqlConnection2 = New System.Data.SqlClient.SqlConnection
        CType(Me.northwindDataSet1, System.ComponentModel.ISupportInitialize).BeginInit()
        CType(Me.gridGroupingControl1, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.SuspendLayout()
        ' 
        ' northwindDataSet1
        ' 
        Me.northwindDataSet1.DataSetName = "DataSet1"
        Me.northwindDataSet1.Locale = New System.Globalization.CultureInfo("en-US")
        ' 
        ' gridGroupingControl1
        ' 
        Me.gridGroupingControl1.DataSource = Me.northwindDataSet1.Orders
        Me.gridGroupingControl1.Dock = System.Windows.Forms.DockStyle.Fill
        Me.gridGroupingControl1.Location = New System.Drawing.Point(0, 0)
        Me.gridGroupingControl1.Name = "gridGroupingControl1"
        Me.gridGroupingControl1.ShowGroupDropArea = True
        Me.gridGroupingControl1.Size = New System.Drawing.Size(690, 664)
        Me.gridGroupingControl1.TabIndex = 0
        ' 
        ' sqlSelectCommand2
        ' 
        Me.sqlSelectCommand2.CommandText = "SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, Shi" + "pVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, Ship" + "Country FROM Orders"
        Me.sqlSelectCommand2.Connection = Me.sqlConnection2
        ' 
        ' sqlInsertCommand2
        ' 
        Me.sqlInsertCommand2.CommandText = "INSERT INTO Orders(CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry) VALUES (@CustomerID, @EmployeeID, @OrderDate, @RequiredDate, @ShippedDate, @ShipVia, @Freight, @ShipName, @ShipAddress, @ShipCity, @ShipRegion, @ShipPostalCode, @ShipCountry); SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry FROM Orders WHERE (OrderID = @@IDENTITY)"
        Me.sqlInsertCommand2.Connection = Me.sqlConnection2
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@CustomerID", System.Data.SqlDbType.NVarChar, 5, "CustomerID"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@EmployeeID", System.Data.SqlDbType.Int, 4, "EmployeeID"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@OrderDate", System.Data.SqlDbType.DateTime, 8, "OrderDate"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@RequiredDate", System.Data.SqlDbType.DateTime, 8, "RequiredDate"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShippedDate", System.Data.SqlDbType.DateTime, 8, "ShippedDate"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipVia", System.Data.SqlDbType.Int, 4, "ShipVia"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Freight", System.Data.SqlDbType.Money, 8, "Freight"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipName", System.Data.SqlDbType.NVarChar, 40, "ShipName"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipAddress", System.Data.SqlDbType.NVarChar, 60, "ShipAddress"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipCity", System.Data.SqlDbType.NVarChar, 15, "ShipCity"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipRegion", System.Data.SqlDbType.NVarChar, 15, "ShipRegion"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, "ShipPostalCode"))
        Me.sqlInsertCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipCountry", System.Data.SqlDbType.NVarChar, 15, "ShipCountry"))
        ' 
        ' sqlUpdateCommand2
        ' 
        Me.sqlUpdateCommand2.CommandText = "UPDATE Orders SET CustomerID = @CustomerID, EmployeeID = @EmployeeID, OrderDate =" + " @OrderDate, RequiredDate = @RequiredDate, ShippedDate = @ShippedDate, ShipVia =" + " @ShipVia, Freight = @Freight, ShipName = @ShipName, ShipAddress = @ShipAddress," + " ShipCity = @ShipCity, ShipRegion = @ShipRegion, ShipPostalCode = @ShipPostalCod" + "e, ShipCountry = @ShipCountry WHERE (OrderID = @Original_OrderID) AND (CustomerI" + "D = @Original_CustomerID OR @Original_CustomerID IS NULL AND CustomerID IS NULL)" + " AND (EmployeeID = @Original_EmployeeID OR @Original_EmployeeID IS NULL AND Empl" + "oyeeID IS NULL) AND (Freight = @Original_Freight OR @Original_Freight IS NULL AN" + "D Freight IS NULL) AND (OrderDate = @Original_OrderDate OR @Original_OrderDate I" + "S NULL AND OrderDate IS NULL) AND (RequiredDate = @Original_RequiredDate OR @Ori" + "ginal_RequiredDate IS NULL AND RequiredDate IS NULL) AND (ShipAddress = @Origina" + "l_ShipAddress OR @Original_ShipAddress IS NULL AND ShipAddress IS NULL) AND (Shi" + "pCity = @Original_ShipCity OR @Original_ShipCity IS NULL AND ShipCity IS NULL) A" + "ND (ShipCountry = @Original_ShipCountry OR @Original_ShipCountry IS NULL AND Shi" + "pCountry IS NULL) AND (ShipName = @Original_ShipName OR @Original_ShipName IS NU" + "LL AND ShipName IS NULL) AND (ShipPostalCode = @Original_ShipPostalCode OR @Orig" + "inal_ShipPostalCode IS NULL AND ShipPostalCode IS NULL) AND (ShipRegion = @Origi" + "nal_ShipRegion OR @Original_ShipRegion IS NULL AND ShipRegion IS NULL) AND (Ship" + "Via = @Original_ShipVia OR @Original_ShipVia IS NULL AND ShipVia IS NULL) AND (S" + "hippedDate = @Original_ShippedDate OR @Original_ShippedDate IS NULL AND ShippedD" + "ate IS NULL); SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, S" + "hippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipP" + "ostalCode, ShipCountry FROM Orders WHERE (OrderID = @OrderID)"
        Me.sqlUpdateCommand2.Connection = Me.sqlConnection2
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@CustomerID", System.Data.SqlDbType.NVarChar, 5, "CustomerID"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@EmployeeID", System.Data.SqlDbType.Int, 4, "EmployeeID"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@OrderDate", System.Data.SqlDbType.DateTime, 8, "OrderDate"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@RequiredDate", System.Data.SqlDbType.DateTime, 8, "RequiredDate"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShippedDate", System.Data.SqlDbType.DateTime, 8, "ShippedDate"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipVia", System.Data.SqlDbType.Int, 4, "ShipVia"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Freight", System.Data.SqlDbType.Money, 8, "Freight"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipName", System.Data.SqlDbType.NVarChar, 40, "ShipName"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipAddress", System.Data.SqlDbType.NVarChar, 60, "ShipAddress"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipCity", System.Data.SqlDbType.NVarChar, 15, "ShipCity"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipRegion", System.Data.SqlDbType.NVarChar, 15, "ShipRegion"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, "ShipPostalCode"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@ShipCountry", System.Data.SqlDbType.NVarChar, 15, "ShipCountry"))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_OrderID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "OrderID", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_CustomerID", System.Data.SqlDbType.NVarChar, 5, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "CustomerID", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_EmployeeID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "EmployeeID", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_Freight", System.Data.SqlDbType.Money, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "Freight", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_OrderDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "OrderDate", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_RequiredDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "RequiredDate", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipAddress", System.Data.SqlDbType.NVarChar, 60, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipAddress", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipCity", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipCity", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipCountry", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipCountry", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipName", System.Data.SqlDbType.NVarChar, 40, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipName", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipPostalCode", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipRegion", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipRegion", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipVia", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipVia", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShippedDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShippedDate", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlUpdateCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@OrderID", System.Data.SqlDbType.Int, 4, "OrderID"))
        ' 
        ' sqlDeleteCommand2
        ' 
        Me.sqlDeleteCommand2.CommandText = "DELETE FROM Orders WHERE (OrderID = @Original_OrderID) AND (CustomerID = @Original_CustomerID OR @Original_CustomerID IS NULL AND CustomerID IS NULL) AND (EmployeeID = @Original_EmployeeID OR @Original_EmployeeID IS NULL AND EmployeeID IS NULL) AND (Freight = @Original_Freight OR @Original_Freight IS NULL AND Freight IS NULL) AND (OrderDate = @Original_OrderDate OR @Original_OrderDate IS NULL AND OrderDate IS NULL) AND (RequiredDate = @Original_RequiredDate OR @Original_RequiredDate IS NULL AND RequiredDate IS NULL) AND (ShipAddress = @Original_ShipAddress OR @Original_ShipAddress IS NULL AND ShipAddress IS NULL) AND (ShipCity = @Original_ShipCity OR @Original_ShipCity IS NULL AND ShipCity IS NULL) AND (ShipCountry = @Original_ShipCountry OR @Original_ShipCountry IS NULL AND ShipCountry IS NULL) AND (ShipName = @Original_ShipName OR @Original_ShipName IS NULL AND ShipName IS NULL) AND (ShipPostalCode = @Original_ShipPostalCode OR @Original_ShipPostalCode IS NULL AND ShipPostalCode IS NULL) AND (ShipRegion = @Original_ShipRegion OR @Original_ShipRegion IS NULL AND ShipRegion IS NULL) AND (ShipVia = @Original_ShipVia OR @Original_ShipVia IS NULL AND ShipVia IS NULL) AND (ShippedDate = @Original_ShippedDate OR @Original_ShippedDate IS NULL AND ShippedDate IS NULL)"
        Me.sqlDeleteCommand2.Connection = Me.sqlConnection2
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_OrderID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "OrderID", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_CustomerID", System.Data.SqlDbType.NVarChar, 5, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "CustomerID", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_EmployeeID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "EmployeeID", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_Freight", System.Data.SqlDbType.Money, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "Freight", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_OrderDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "OrderDate", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_RequiredDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "RequiredDate", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipAddress", System.Data.SqlDbType.NVarChar, 60, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipAddress", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipCity", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipCity", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipCountry", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipCountry", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipName", System.Data.SqlDbType.NVarChar, 40, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipName", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipPostalCode", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipRegion", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipRegion", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShipVia", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShipVia", System.Data.DataRowVersion.Original, Nothing))
        Me.sqlDeleteCommand2.Parameters.Add(New System.Data.SqlClient.SqlParameter("@Original_ShippedDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, False, CType(0, System.Byte), CType(0, System.Byte), "ShippedDate", System.Data.DataRowVersion.Original, Nothing))
        ' 
        ' sqlDataAdapterOrders
        ' 
        Me.sqlDataAdapterOrders.DeleteCommand = Me.sqlDeleteCommand2
        Me.sqlDataAdapterOrders.InsertCommand = Me.sqlInsertCommand2
        Me.sqlDataAdapterOrders.SelectCommand = Me.sqlSelectCommand2
        Me.sqlDataAdapterOrders.TableMappings.AddRange(New System.Data.Common.DataTableMapping() {New System.Data.Common.DataTableMapping("Table", "Orders", New System.Data.Common.DataColumnMapping() {New System.Data.Common.DataColumnMapping("OrderID", "OrderID"), New System.Data.Common.DataColumnMapping("CustomerID", "CustomerID"), New System.Data.Common.DataColumnMapping("EmployeeID", "EmployeeID"), New System.Data.Common.DataColumnMapping("OrderDate", "OrderDate"), New System.Data.Common.DataColumnMapping("RequiredDate", "RequiredDate"), New System.Data.Common.DataColumnMapping("ShippedDate", "ShippedDate"), New System.Data.Common.DataColumnMapping("ShipVia", "ShipVia"), New System.Data.Common.DataColumnMapping("Freight", "Freight"), New System.Data.Common.DataColumnMapping("ShipName", "ShipName"), New System.Data.Common.DataColumnMapping("ShipAddress", "ShipAddress"), New System.Data.Common.DataColumnMapping("ShipCity", "ShipCity"), New System.Data.Common.DataColumnMapping("ShipRegion", "ShipRegion"), New System.Data.Common.DataColumnMapping("ShipPostalCode", "ShipPostalCode"), New System.Data.Common.DataColumnMapping("ShipCountry", "ShipCountry")})})
        Me.sqlDataAdapterOrders.UpdateCommand = Me.sqlUpdateCommand2
        ' 
        ' sqlConnection2
        ' 
        Me.sqlConnection2.ConnectionString = "data source=(local)\NETSDK;initial catalog=Northwind;integrated security=SSPI;per" + "sist security info=True;workstation id=localhost;packet size=4096"
        ' 
        ' 
        ' Form1
        ' 
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(690, 664)
        Me.Controls.Add(gridGroupingControl1)
        Me.Name = "Form1"
        Me.Text = "Syncfusion Essential Grid - SummaryInCaption"
        CType(Me.northwindDataSet1, System.ComponentModel.ISupportInitialize).EndInit()
        CType(Me.gridGroupingControl1, System.ComponentModel.ISupportInitialize).EndInit()
        Me.ResumeLayout(False)
    End Sub 'InitializeComponent 
#End Region


    '/ <summary>
    '/ The main entry point for the application.
    '/ </summary>
    <STAThread()> _
    Shared Sub Main()

#If ((SyncfusionFramework1_1 OrElse SyncfusionFramework2_0)) Then
        Application.EnableVisualStyles()
#End If

        Application.Run(New Form1)
    End Sub 'Main

#Region "Unbound Fields and Covered Ranges"


    Private Sub ordersDescriptor_QueryValue(ByVal sender As Object, ByVal e As FieldValueEventArgs)
        ' This methods maps from the underlying value to a related value
        Dim obj As Object = e.Record.GetValue("ShipVia")
        If Not (obj Is Nothing) AndAlso Not TypeOf obj Is DBNull Then
            Dim shipVia As Integer = Fix(obj)
            Select Case shipVia
                Case 0
                    e.Value = "null"
                Case 1
                    e.Value = "One"
                Case 2
                    e.Value = "Two"
                Case 3
                    e.Value = "Three"
            End Select
        End If
    End Sub 'ordersDescriptor_QueryValue

    Private captionCoverCols As Integer


    Private Sub TableModel_QueryCoveredRange(ByVal sender As Object, ByVal e As GridQueryCoveredRangeEventArgs)
        Dim thisTable As GridTable = Me.gridGroupingControl1.Table
        If e.RowIndex < thisTable.DisplayElements.Count Then
            Dim el As Element = thisTable.DisplayElements(e.RowIndex)

            Select Case el.Kind
                Case DisplayElementKind.Caption
                    ' Cover some cells of the caption bar (specified with captionCover)
                    Dim gs As IGridGroupOptionsSource = el.ParentGroup '
                    If Not (gs Is Nothing) AndAlso gs.GroupOptions.ShowCaptionSummaryCells Then
                        Dim startCol As Integer = el.GroupLevel + 1
                        If Not gs.GroupOptions.ShowCaptionPlusMinus Then
                            startCol -= 1
                        End If
                        If e.ColIndex >= startCol AndAlso e.ColIndex <= startCol + Me.captionCoverCols Then
                            e.Range = GridRangeInfo.Cells(e.RowIndex, startCol, e.RowIndex, startCol + Me.captionCoverCols)
                            e.Handled = True
                        End If
                    End If
                    Exit Select
            End Select
        End If
    End Sub 'TableModel_QueryCoveredRange


#End Region

End Class 'Form1
